# Proposed redesign of basis_checker_one_column() from pg/macros/MatrixCheckers.pl
# to overcome problems with the behavior of that function.

# The original code of pg/macros/MatrixCheckers.pl is
# by Paul Pearson, Hope College, Department of Mathematics
# and was the original basis and the inspiration for much
# of that is in the proposed new checker.

# For not the proposed new checker is called basis_checker_one_column_tani()
# and the local version of basis_checker_one_column() has a bit of debugging
# code added.

sub _MatrixCheckers_init {}; # don't reload this file

loadMacros("MathObjects.pl");

################################################

sub basis_checker_one_column_tani {

      my ( $correct, $student, $answerHash ) = @_;

      # Most of the answer checking is done on integers
      # or on decimals like 0.24381729, so we will set the
      # tolerance accordingly in a local context.
      my $context = Context()->copy;
      $context->flags->set(
        tolerance => 0.0000001,
        tolType => "absolute",
      );

      #my $C = Matrix($context,$correct);
      #my $S = Matrix($context,$student);
      my $C = Vector($context,$correct);
      my $S = Vector($context,$student);

      # This test is overly permissive in terms of small shifts being ignored
      my $test1 = $S->isParallel($C);

      if ( ! $test1 ) {
        return 0;
      }

      # If the prior test passed, use a rank based test
      my $M1 = Matrix( [ 
            Matrix($context,$correct)->transpose->row(1), 
            Matrix($context,$student)->transpose->row(1),
         ] ) ;
      my $rank1 = rank( $M1 );
      #warn "M1 = $M1, r1 = $rank1";
      if ( $rank1 == 1 ) {
        return 1;
      } else {
        return 0;
      }
}


##########################################

sub basis_checker_one_column { 

      my ( $correct, $student, $answerHash ) = @_;
      
      # Most of the answer checking is done on integers 
      # or on decimals like 0.24381729, so we will set the
      # tolerance accordingly in a local context.
      my $context = Context()->copy;
      $context->flags->set(
        tolerance => 0.001,
        tolType => "absolute",
      );

      #my $C = Matrix($context,$correct);
      #my $S = Matrix($context,$student);
      my $C = Vector($context,$correct);
      my $S = Vector($context,$student);
      return $S->isParallel($C);

}

